/* -*- Mode: C -*- */

/* COPYING
 * Liquid Rendering Library
 * (c) 2009 by Henrik Theiling
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LIQUID_SYSTYPE_H_129323479844
#define LIQUID_SYSTYPE_H_129323479844

#include <liquid/config.h>

/* FIXME:
 * This assumes that a char is 8 bits.  If that is untrue on
 * your architecture, please provide the types yourself.
 *
 * FIXME:
 * For Modern C++, we might want to provide functions
 * supporting char16_t, char32_t and wchar_t all at the same
 * time and doing the right thing(TM).  For that, we'll need
 * to improve this.
 */

/* Determine character data types */
/* 8 bit */
#ifndef LIQ_CHAR8
#define LIQ_CHAR8 char
#endif

/* 16 bit */
#ifndef LIQ_CHAR16
#  if defined(LIQ_SIZEOF_CHAR16_T) && LIQ_SIZEOF_CHAR16_T == 2
#    define LIQ_CHAR16             char16_t
#    define LIQ_CHAR16_IS_CHAR16_T
#    define LIQ_STR16(X)           u##X
#  elif defined(LIQ_SIZEOF_WCHAR_T) && LIQ_SIZEOF_WCHAR_T == 2
#    define LIQ_CHAR16             wchar_t
#    define LIQ_CHAR16_IS_WCHAR_T
#    define LIQ_STR16(X)           L##X
#  else
#    define LIQ_CHAR16 unsigned short
#  endif
#endif

/* 32 bit */
#ifndef LIQ_CHAR32
#  if defined(LIQ_SIZEOF_CHAR32_T) && LIQ_SIZEOF_CHAR32_T == 4
#    define LIQ_CHAR32             char32_t
#    define LIQ_CHAR32_IS_CHAR32_T
#    define LIQ_STR32(X)           U##X
#  elif defined(LIQ_SIZEOF_WCHAR_T) && LIQ_SIZEOF_WCHAR_T == 4
#    define LIQ_CHAR32             wchar_t
#    define LIQ_CHAR32_IS_WCHAR_T
#    define LIQ_STR32(X)           L##X
#  elif defined(LIQ_SIZEOF_INT) && LIQ_SIZEOF_INT == 4
#    define LIQ_CHAR32 unsigned int
#  elif defined(LIQ_SIZEOF_LONG) && LIQ_SIZEOF_LONG == 4
#    define LIQ_CHAR32 unsigned long
#  else
#    define LIQ_CHAR32 unsigned int
#  endif
#endif

/* 32-bit int type */
#ifndef LIQ_UINT32
#  if defined(LIQ_SIZEOF_INT) && LIQ_SIZEOF_INT == 4
#    define LIQ_UINT32 unsigned int
#    define LIQ_INT32  signed int
#  elif defined(LIQ_SIZEOF_LONG) && LIQ_SIZEOF_LONG == 4
#    define LIQ_UINT32 unsigned long
#    define LIQ_INT32  signed long
#  else
#    define LIQ_UINT32 unsigned int
#    define LIQ_INT32  signed int
#  endif
#endif
#ifndef LIQ_UINT32_C
#  define LIQ_UINT32_C(X) ((LiqUInt32)(X ## UL))
#endif

/* Types used for bitfields */
#ifdef LIQ_BIT_TYPES
#  ifndef LIQ_UBITS8
#  define LIQ_UBITS8 LiqUInt8
#  endif
#  ifndef LIQ_SBITS8
#  define LIQ_SBITS8 LiqInt8
#  endif
#  ifndef LIQ_UBITS16
#  define LIQ_UBITS16 LiqUInt16
#  endif
#  ifndef LIQ_SBITS16
#  define LIQ_SBITS16 LiqInt16
#  endif
#  ifndef LIQ_UBITS32
#  define LIQ_UBITS32 LiqUInt32
#  endif
#  ifndef LIQ_SBITS32
#  define LIQ_SBITS32 LiqInt32
#  endif
#else
#  ifndef LIQ_UBITS8
#  define LIQ_UBITS8 unsigned int
#  endif
#  ifndef LIQ_SBITS8
#  define LIQ_SBITS8 signed int
#  endif
#  ifndef LIQ_UBITS16
#  define LIQ_UBITS16 unsigned int
#  endif
#  ifndef LIQ_SBITS16
#  define LIQ_SBITS16 signed int
#  endif
#  ifndef LIQ_UBITS32
#  define LIQ_UBITS32 unsigned int
#  endif
#  ifndef LIQ_SBITS32
#  define LIQ_SBITS32 signed int
#  endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef LIQ_CHAR8      LiqChar8;
typedef LIQ_CHAR16     LiqChar16;
typedef LIQ_CHAR32     LiqChar32;
typedef unsigned char  LiqUInt8;
typedef unsigned short LiqUInt16;
typedef LIQ_UINT32     LiqUInt32;
typedef signed char    LiqInt8;
typedef signed short   LiqInt16;
typedef LIQ_INT32      LiqInt32;

/* There's no easy way to use C99's _Bool and mix it with C++'s bool.
 * There is guarantee that the type will behave the same in the ABI,
 * and although it would be mad if the ABI was different, I'm still
 * not going to try to depend on its working.
 * (gcc does provide stdbool.h for both C and C++ and seems the
 * types are interoperable, but gcc's not the all there is.)
 *
 * Further, for bool, I'd need another configure check, which
 * is overkill.
 *
 * So we'll have our own boolean and use the old !! idiom for
 * casting to 0/1 if needed. */
typedef unsigned char LiqBool;
#define LIQ_FALSE 0
#define LIQ_TRUE  1

#ifdef __GNUC__
#  define LIQ_UNUSED     __attribute__((__unused__))
#  define LIQ_DEPRECATED __attribute__((__deprecated__))
#  define LIQ_NORETURN   __attribute__((__noreturn__))
#elif defined(_MSC_VER)
#  define LIQ_UNUSED
#  define LIQ_DEPRECATED _declspec(unused)
#  define LIQ_NORETURN   _declspec(noreturn)
#else
#  define LIQ_UNUSED
#  define LIQ_DEPRECATED
#  define LIQ_NORETURN
#endif

#ifdef __cplusplus
}
#endif

#endif /* LIQUID_SYSTYPE_H_129323479844 */
